_debug = 0
try:
    import timeoutsocket # http://www.timo-tasi.org/python/timeoutsocket.py
    timeoutsocket.setDefaultSocketTimeout(10)
except ImportError:
    pass
try:
    import xmlrpclib # http://www.pythonware.com/products/xmlrpc/
except ImportError:
    pass
from sgmllib import SGMLParser
import urllib, urlparse, re, sys

class BaseParser(SGMLParser):
    def __init__(self, baseuri):
        SGMLParser.__init__(self)
        self.links = []
        self.baseuri = baseuri
        
class LinkParser(BaseParser):
    RSSTYPE = 'application/rss+xml'
    def do_link(self, attrs):
        if not ('rel', 'alternate') in attrs: return
        types = [v for k,v in attrs if k=='type']
        if not types: return
        type = types[0]
        if type[:len(self.RSSTYPE)] != self.RSSTYPE: return
        hrefs = [v for k,v in attrs if k=='href']
        if not hrefs: return
        self.links.append(urlparse.urljoin(self.baseuri, hrefs[0]))

class ALinkParser(BaseParser):
    def start_a(self, attrs):
        hrefs = [v for k,v in attrs if k=='href']
        if not hrefs: return
        self.links.append(urlparse.urljoin(self.baseuri, hrefs[0]))

def makeFullURI(uri):
    if not uri.count('http://'):
        uri = 'http://%s' % uri
    return uri

def getLinks(data, baseuri):
    p = LinkParser(baseuri)
    p.feed(data)
    return p.links

def getALinks(data, baseuri):
    p = ALinkParser(baseuri)
    p.feed(data)
    return p.links

def getLocalLinks(links, baseuri):
    baseuri = baseuri.lower()
    urilen = len(baseuri)
    return [l for l in links if l[:urilen].lower() == baseuri]

def isFeedLink(link):
    return link[-4:].lower() in ('.rss', '.rdf', '.xml')

def isXMLRelatedLink(link):
    link = link.lower()
    return link.count('rss') + link.count('rdf') + link.count('xml')

def isRSS(data):
    data = data.lower()
    return data.count('<rss') + data.count('<rdf')

def isFeed(uri):
    if _debug: print 'verifying that %s is a feed' % uri
    usock = urllib.urlopen(uri)
    data = usock.read(255)
    usock.close()
    return isRSS(data)

def getFeedsFromSyndic8(uri):
    if not sys.modules.has_key('xmlrpclib'): return []
    feedids = server.syndic8.FindFeeds(uri)
    infolist = server.syndic8.GetFeedInfo(feedids)
    feeds = [f['dataurl'] for f in infolist if f['status']=='Syndicated']
    if _debug: print 'found %s feeds through Syndic8' % len(feeds)
    return feeds
    
def getFeeds(shorturi):
    uri = makeFullURI(shorturi)
    usock = urllib.urlopen(uri)
    data = usock.read()
    usock.close()
    # is this already a feed?
    if isRSS(data):
        return [uri]
    # nope, it's a page, try LINK tags first
    if _debug: print 'looking for LINK tags'
    feeds = getLinks(data, uri)
    if _debug: print 'found %s feeds through LINK tags' % len(feeds)
    feeds = filter(isFeed, feeds)
    if not feeds:
        # no LINK tags, look for regular <A> links that point to feeds
        if _debug: print 'no LINK tags, looking at A tags'
        links = getALinks(data, uri)
        locallinks = getLocalLinks(links, uri)
        feeds = filter(isFeedLink, locallinks)
        if not feeds:
            feeds = filter(isXMLRelatedLink, locallinks)
        if not feeds:
            feeds = filter(isXMLRelatedLink, links)
        feeds = filter(isFeed, feeds)
    if not feeds:
        # still no luck, search Syndic8 for feeds (requires xmlrpclib)
        if _debug: print 'still no luck, searching Syndic8'
        feeds = getFeedsFromSyndic8(shorturi)
    return feeds

if __name__ == '__main__':
    import pprint
    if sys.argv[1:]:
        uri = sys.argv[1]
    else:
        uri = 'http://diveintomark.org/'
    pprint.pprint(getFeeds(uri))
